@@ -4,7 +4,8 @@
  * Copyright (C) 1997-2003 Marko Macek
  */
 #include "config.h"
-
+#define WMAPP
+#include "appnames.h"
 #include "yfull.h"
 #include "wmprog.h"
 #include "wmwinmenu.h"
@@ -25,7 +26,7 @@
 #include "yprefs.h"
 #include "prefs.h"
 #include "udir.h"
-#include "appnames.h"
+#include "ascii.h"
 #include "ypaths.h"
 #include "yxcontext.h"
 #ifdef CONFIG_XFREETYPE
@@ -34,14 +35,15 @@
 #endif
 #undef override
 #include <X11/Xproto.h>
+#include <wordexp.h>
 #include "intl.h"
 
 char const *ApplicationName("IceWM");
 RebootShutdown rebootOrShutdown = Logout;
 static bool initializing(true);
 
-YWMApp *wmapp(NULL);
-YWindowManager *manager(NULL);
+YWMApp *wmapp;
+YWindowManager *manager;
 
 YCursor YWMApp::sizeRightPointer;
 YCursor YWMApp::sizeTopRightPointer;
@@ -62,8 +64,6 @@ lazily<SharedWindowList> windowListMenu;
 lazy<LogoutMenu> logoutMenu;
 lazily<RootMenu> rootMenu;
 
-static ref<YIcon> defaultAppIcon;
-
 static bool replace_wm;
 static bool post_preferences;
 static bool show_extensions;
@@ -110,14 +110,14 @@ static Window registerProtocols1(char **argv, int argc) {
     static char wm_instance[] = "icewm";
 
     XClassHint class_hint = {
-        (argv == NULL) ? wm_instance : NULL,
+        argv ? nullptr : wm_instance,
         wm_class
     };
 
     static char wm_name[] = "IceWM " VERSION " (" HOSTOS "/" HOSTCPU ")";
 
-    Xutf8SetWMProperties(xapp->display(), xid, wm_name, NULL,
-            argv, argc, NULL, NULL, &class_hint);
+    Xutf8SetWMProperties(xapp->display(), xid, wm_name, nullptr,
+            argv, argc, nullptr, nullptr, &class_hint);
 
     XClientMessageEvent ev;
 
@@ -328,16 +328,21 @@ static void registerProtocols2(Window xid) {
     registerNetProperties(xid);
 }
 
-static void unregisterProtocols() {
-    XDeleteProperty(xapp->display(),
-                    manager->handle(),
-                    _XA_WIN_PROTOCOLS);
+void YWMApp::unregisterProtocols() {
+    if (managerWindow) {
+        YAtom wmSx("WM_S", true);
+        if (managerWindow == XGetSelectionOwner(xapp->display(), wmSx)) {
+            XSelectInput(xapp->display(), xapp->root(), None);
+            XDeleteProperty(xapp->display(), xapp->root(), _XA_WIN_PROTOCOLS);
+            XSetSelectionOwner(xapp->display(), wmSx, None, CurrentTime);
+        }
+        managerWindow = None;
+        xapp->sync();
+    }
 }
 
 void YWMApp::initIconSize() {
-    XIconSize *is;
-
-    is = XAllocIconSize();
+    XIconSize *is = XAllocIconSize();
     if (is) {
         is->min_width = 32;
         is->min_height = 32;
@@ -350,125 +355,24 @@ void YWMApp::initIconSize() {
     }
 }
 
-static void initFontPath(IApp *app) {
-    if (themeName) { // =================== find the current theme directory ===
-        upath themesFile(themeName);
-        upath themesDir = themesFile.parent();
-        upath fonts_dirFile = themesDir.child("fonts.dir");
-        upath fonts_dirPath = app->findConfigFile(fonts_dirFile);
-        upath fonts_dirDir(null);
-
-        if (fonts_dirPath != null)
-            upath fonts_dirDir = fonts_dirPath.parent();
-
-#if 0
-        char themeSubdir[PATH_MAX];
-        strncpy(themeSubdir, themeName, sizeof(themeSubdir) - 1);
-        themeSubdir[sizeof(themeSubdir) - 1] = '\0';
-
-        char * strfn(strrchr(themeSubdir, '/'));
-        if (strfn) *strfn = '\0';
-
-        // ================================ is there a file named fonts.dir? ===
-        upath fontsdir;
-
-        if (*themeName == '/')
-            fontsdir = cstrJoin(themeSubdir, "/fonts.dir", NULL);
-        else {
-            strfn = cstrJoin("themes/", themeSubdir, "/fonts.dir", NULL);
-            fontsdir = (app->findConfigFile(strfn));
-            delete[] strfn;
-        }
-#endif
-
-        if (fonts_dirDir != null) { // =========================== build a new font path ===
-            cstring dir(fonts_dirDir.path());
-            const char *fontsdir = dir.c_str();
-
-#ifdef CONFIG_XFREETYPE
-            MSG(("font dir add %s", fontsdir));
-            FcConfigAppFontAddDir(0, (FcChar8 *)fontsdir);
-#endif
-#ifdef CONFIG_COREFONTS
-
-            int ndirs; // ------------------- retrieve the old X's font path ---
-            char ** fontPath(XGetFontPath(xapp->display(), &ndirs));
-
-            char ** newFontPath = new char *[ndirs + 1];
-            newFontPath[ndirs] = (char *)fontsdir;
-
-            if (fontPath)
-                memcpy(newFontPath, fontPath, ndirs * sizeof (char *));
-            else
-                warn(_("Unable to get current font path."));
-
-#ifdef DEBUG
-            for (int n = 0; n < ndirs + 1; ++n)
-                MSG(("Font path element %d: %s", n, newFontPath[n]));
-#endif
-
-            char * icewmFontPath; // ---------- find death icewm's font path ---
-            Atom r_type; int r_format;
-            unsigned long count, bytes_remain;
-
-            if (XGetWindowProperty(xapp->display(),
-                                   manager->handle(),
-                                   _XA_ICEWM_FONT_PATH,
-                                   0, PATH_MAX, False, XA_STRING,
-                                   &r_type, &r_format,
-                                   &count, &bytes_remain,
-                                   (unsigned char **) &icewmFontPath) ==
-                Success && icewmFontPath) {
-                if (r_type == XA_STRING && r_format == 8) {
-                    for (int n(ndirs - 1); n > 0; --n) // ---- remove death paths ---
-                        if (!strcmp(icewmFontPath, newFontPath[n])) {
-                            memmove(newFontPath + n, newFontPath + n + 1,
-                                    (ndirs - n) * sizeof(char *));
-                            --ndirs;
-                        }
-                } else
-                    warn(_("Unexpected format of ICEWM_FONT_PATH property"));
-
-                XFree(icewmFontPath);
-            }
-
-#ifdef DEBUG
-            for (int n = 0; n < ndirs + 1; ++n)
-                MSG(("Font path element %d: %s", n, newFontPath[n]));
-#endif
-            // ----------------------------------------- set the new font path ---
-            XChangeProperty(xapp->display(), manager->handle(),
-                            _XA_ICEWM_FONT_PATH, XA_STRING, 8, PropModeReplace,
-                            (unsigned char *) fontsdir, strlen(fontsdir));
-            XSetFontPath(xapp->display(), newFontPath, ndirs + 1);
-
-            if (fontPath) XFreeFontPath(fontPath);
-            delete[] newFontPath;
-#endif
-        }
-    }
-}
-
 void YWMApp::initIcons() {
     defaultAppIcon = YIcon::getIcon("app");
 }
-void YWMApp::termIcons() {
-    defaultAppIcon = null;
-}
+
 ref<YIcon> YWMApp::getDefaultAppIcon() {
     return defaultAppIcon;
 }
 
 CtrlAltDelete* YWMApp::getCtrlAltDelete() {
-    if (ctrlAltDelete == 0) {
+    if (ctrlAltDelete == nullptr) {
         ctrlAltDelete = new CtrlAltDelete(this, manager);
     }
     return ctrlAltDelete;
 }
 
 SwitchWindow* YWMApp::getSwitchWindow() {
-    if (switchWindow == 0 && quickSwitch) {
-        switchWindow = new SwitchWindow(manager, NULL, quickSwitchVertical);
+    if (switchWindow == nullptr && quickSwitch) {
+        switchWindow = new SwitchWindow(manager, nullptr, quickSwitchVertical);
     }
     return switchWindow;
 }
@@ -495,28 +399,28 @@ void LogoutMenu::updatePopup() {
         return;
 
     if (showLogoutMenu) {
-        logoutMenu->setShared(true); /// !!! get rid of this (refcount objects)
+        setShared(true); /// !!! get rid of this (refcount objects)
         if (showLogoutSubMenu) {
-            logoutMenu->addItem(_("_Logout"), -2, null, actionLogout)->setChecked(true);
-            logoutMenu->addItem(_("_Cancel logout"), -2, null, actionCancelLogout)->setEnabled(false);
-            logoutMenu->addSeparator();
+            addItem(_("_Logout"), -2, null, actionLogout)->setChecked(true);
+            addItem(_("_Cancel logout"), -2, null, actionCancelLogout)->setEnabled(false);
+            addSeparator();
 
-            int const oldItemCount = logoutMenu->itemCount();
+            int const oldItemCount = itemCount();
             if (canLock())
-                logoutMenu->addItem(_("Lock _Workstation"), -2, null, actionLock, "lock");
+                addItem(_("Lock _Workstation"), -2, null, actionLock, "lock");
             if (canShutdown(Reboot))
-                logoutMenu->addItem(_("Re_boot"), -2, null, actionReboot, "reboot");
+                addItem(_("Re_boot"), -2, null, actionReboot, "reboot");
             if (canShutdown(Shutdown))
-                logoutMenu->addItem(_("Shut_down"), -2, null, actionShutdown, "shutdown");
+                addItem(_("Shut_down"), -2, null, actionShutdown, "shutdown");
             if (couldRunCommand(suspendCommand))
-                logoutMenu->addItem(_("_Sleep mode"), -2, null, actionSuspend, "suspend");
+                addItem(_("_Sleep mode"), -2, null, actionSuspend, "suspend");
 
-            if (logoutMenu->itemCount() != oldItemCount)
-                logoutMenu->addSeparator();
+            if (itemCount() != oldItemCount)
+                addSeparator();
 
-            logoutMenu->addItem(_("Restart _Icewm"), -2, null, actionRestart, "restart");
+            addItem(_("Restart _Icewm"), -2, null, actionRestart, "restart");
 
-            logoutMenu->addItem(_("Restart _Xterm"), -2, null, actionRestartXterm, TERM);
+            addItem(_("Restart _Xterm"), -2, null, actionRestartXterm, TERM);
 
         }
     }
@@ -526,13 +430,13 @@ void LayerMenu::updatePopup() {
     if (itemCount())
         return;
 
-    layerMenu->addItem(_("_Menu"),       -2, null, layerActionSet[WinLayerMenu]);
-    layerMenu->addItem(_("_Above Dock"), -2, null, layerActionSet[WinLayerAboveDock]);
-    layerMenu->addItem(_("_Dock"),       -2, null, layerActionSet[WinLayerDock]);
-    layerMenu->addItem(_("_OnTop"),      -2, null, layerActionSet[WinLayerOnTop]);
-    layerMenu->addItem(_("_Normal"),     -2, null, layerActionSet[WinLayerNormal]);
-    layerMenu->addItem(_("_Below"),      -2, null, layerActionSet[WinLayerBelow]);
-    layerMenu->addItem(_("D_esktop"),    -2, null, layerActionSet[WinLayerDesktop]);
+    addItem(_("_Menu"),       -2, null, layerActionSet[WinLayerMenu]);
+    addItem(_("_Above Dock"), -2, null, layerActionSet[WinLayerAboveDock]);
+    addItem(_("_Dock"),       -2, null, layerActionSet[WinLayerDock]);
+    addItem(_("_OnTop"),      -2, null, layerActionSet[WinLayerOnTop]);
+    addItem(_("_Normal"),     -2, null, layerActionSet[WinLayerNormal]);
+    addItem(_("_Below"),      -2, null, layerActionSet[WinLayerBelow]);
+    addItem(_("D_esktop"),    -2, null, layerActionSet[WinLayerDesktop]);
 }
 
 void MoveMenu::updatePopup() {
@@ -542,7 +446,7 @@ void MoveMenu::updatePopup() {
     for (int w = 1; w <= workspaceCount; w++) {
         char s[128];
         snprintf(s, sizeof s, "%2d.  %s ", w, workspaceNames[w - 1]);
-        moveMenu->addItem(s, 1,
+        addItem(s, 1,
                 w ==  1 ? KEY_NAME(gKeySysWorkspace1TakeWin)  :
                 w ==  2 ? KEY_NAME(gKeySysWorkspace2TakeWin)  :
                 w ==  3 ? KEY_NAME(gKeySysWorkspace3TakeWin)  :
@@ -630,12 +534,12 @@ YMenu* YWMApp::getWindowMenu() {
 bool YWMApp::handleTimer(YTimer *timer) {
     if (timer == errorTimer) {
         errorTimer = null;
-        if (errorRequestCode == X_SetInputFocus && errorFrame != 0) {
+        if (errorRequestCode == X_SetInputFocus && errorFrame != nullptr) {
             if (errorFrame == manager->getFocus()) {
                 if (errorFrame->client()) {
                     errorFrame->client()->testDestroyed();
                 }
-                manager->setFocus(0);
+                manager->setFocus(nullptr);
                 manager->focusLastWindow();
             }
         }
@@ -683,10 +587,10 @@ int YWMApp::handleError(XErrorEvent *xev) {
 #ifdef DEBUG
 void dumpZorder(const char *oper, YFrameWindow *w, YFrameWindow *a) {
     YFrameWindow *p = manager->top(w->getActiveLayer());
-    msg("---- %s ", oper);
+    msg("--- %s ", oper);
     while (p) {
         if (p && p->client()) {
-            cstring cs(p->client()->windowTitle());
+            mstring cs(p->client()->windowTitle());
             msg(" %c %c 0x%lX: %s", (p == w) ? '*' : ' ',  (p == a) ? '#' : ' ', p->client()->handle(), cs.c_str());
         } else
             msg("?? 0x%lX", p->handle());
@@ -712,11 +616,11 @@ void YWMApp::runRestart(const char *path, char *const *args) {
         if (args) {
             execvp(path, args);
         } else {
-            execlp(path, path, (void *)NULL);
+            execlp(path, path, nullptr);
         }
     } else {
         if (mainArgv[0][0] == '/' ||
-            (strchr(mainArgv[0], '/') != 0 &&
+            (strchr(mainArgv[0], '/') != nullptr &&
              access(mainArgv[0], X_OK) == 0))
         {
             execv(mainArgv[0], mainArgv);
@@ -735,7 +639,7 @@ void YWMApp::runRestart(const char *path, char *const *args) {
 void YWMApp::restartClient(const char *cpath, char *const *cargs) {
     csmart path(newstr(cpath));
     YStringArray sargs((const char**) cargs);
-    char *const *args = (cargs == 0) ? 0 : sargs.getCArray();
+    char *const *args = (cargs == nullptr) ? nullptr : sargs.getCArray();
 
     wmapp->signalGuiEvent(geRestart);
     manager->unmanageClients();
@@ -743,12 +647,31 @@ void YWMApp::restartClient(const char *cpath, char *const *cargs) {
 
     runRestart(path, args);
 
+    // icesm knows how to restart.
+    if (notifyParent && notifiedParent && kill(notifiedParent, 0) == 0)
+        ::exit(ICESM_EXIT_RESTART);
+
     /* somehow exec failed, try to recover */
-    managerWindow = registerProtocols1(NULL, 0);
+    managerWindow = registerProtocols1(mainArgv, mainArgc);
     registerProtocols2(managerWindow);
     manager->manageClients();
 }
 
+int YWMApp::runProgram(const char *path, const char *const *args) {
+    mstring command;
+    YTraceProg trace;
+    if (trace.tracing()) {
+        command = path;
+        if (nonempty(*args)) {
+            for (int i = 1; args[i]; ++i) {
+                command = mstring(command, " ", args[i]);
+            }
+        }
+        trace.init(command);
+    }
+    return YApplication::runProgram(path, args);
+}
+
 void YWMApp::runOnce(const char *resource, long *pid,
                      const char *path, char *const *args)
 {
@@ -761,6 +684,21 @@ void YWMApp::runOnce(const char *resource, long *pid,
     *pid = runProgram(path, args);
 }
 
+void YWMApp::runCommand(const char *cmdline) {
+    const char shell[] = "&();<>`{}|";
+    wordexp_t exp = {};
+    if (strpbrk(cmdline, shell) == nullptr &&
+        wordexp(cmdline, &exp, WRDE_NOCMD) == 0)
+    {
+        runProgram(exp.we_wordv[0], exp.we_wordv);
+        wordfree(&exp);
+    }
+    else {
+        char const * argv[] = { "/bin/sh", "-c", cmdline, nullptr };
+        runProgram(argv[0], argv);
+    }
+}
+
 void YWMApp::runCommandOnce(const char *resource, const char *cmdline, long *pid) {
     if (0 < *pid && mapClientByPid(resource, *pid))
         return;
@@ -768,9 +706,18 @@ void YWMApp::runCommandOnce(const char *resource, const char *cmdline, long *pid
     if (mapClientByResource(resource, pid))
         return;
 
-    char const *const argv[] = { "/bin/sh", "-c", cmdline, NULL };
-
-    *pid = runProgram(argv[0], (char *const *) argv);
+    const char shell[] = "&();<>`{}|";
+    wordexp_t exp = {};
+    if (strpbrk(cmdline, shell) == nullptr &&
+        wordexp(cmdline, &exp, WRDE_NOCMD) == 0)
+    {
+        *pid = runProgram(exp.we_wordv[0], exp.we_wordv);
+        wordfree(&exp);
+    }
+    else {
+        char const *const argv[] = { "/bin/sh", "-c", cmdline, nullptr };
+        *pid = runProgram(argv[0], argv);
+    }
 }
 
 bool YWMApp::mapClientByPid(const char* resource, long pid) {
@@ -821,20 +768,18 @@ void YWMApp::setFocusMode(FocusModels mode) {
 }
 
 void YWMApp::actionPerformed(YAction action, unsigned int /*modifiers*/) {
-
-
     if (action == actionLogout) {
         doLogout(Logout);
     } else if (action == actionCancelLogout) {
         cancelLogout();
     } else if (action == actionLock) {
-        this->runCommand(lockCommand);
+        runCommand(lockCommand);
     } else if (action == actionShutdown) {
-        manager->doWMAction(ICEWM_ACTION_SHUTDOWN);
+        handleSMAction(ICEWM_ACTION_SHUTDOWN);
     } else if (action == actionSuspend) {
-        manager->doWMAction(ICEWM_ACTION_SUSPEND);
+        handleSMAction(ICEWM_ACTION_SUSPEND);
     } else if (action == actionReboot) {
-        manager->doWMAction(ICEWM_ACTION_REBOOT);
+        handleSMAction(ICEWM_ACTION_REBOOT);
     } else if (action == actionRestart) {
 #if defined(DEBUG) || defined(PRECON)
         // Prefer a return from main for cleanup checking; icesm restarts.
@@ -842,7 +787,7 @@ void YWMApp::actionPerformed(YAction action, unsigned int /*modifiers*/) {
             this->exit(ICESM_EXIT_RESTART);
         else
 #endif
-            restartClient(0, 0);
+            restartClient(nullptr, nullptr);
     }
     else if (action == actionRestartXterm) {
         struct t_executor : public YMsgBoxListener {
@@ -879,14 +824,13 @@ void YWMApp::actionPerformed(YAction action, unsigned int /*modifiers*/) {
     } else if (action == actionRefresh) {
         osmart<YWindow> w(new YWindow());
         if (w) {
-            w->setGeometry(YRect(0, 0,
-                                 desktop->width(), desktop->height()));
+            w->setGeometry(desktop->geometry());
             w->raise();
             w->show();
             w->hide();
         }
     } else if (action == actionAbout) {
-        if (aboutDlg == 0)
+        if (aboutDlg == nullptr)
             aboutDlg = new AboutDlg();
         else
             aboutDlg->getFrame()->setWorkspace(manager->activeWorkspace());
@@ -895,7 +839,7 @@ void YWMApp::actionPerformed(YAction action, unsigned int /*modifiers*/) {
     } else if (action == actionTileVertical ||
                action == actionTileHorizontal)
     {
-        YFrameWindow **w = 0;
+        YFrameWindow **w = nullptr;
         int count = 0;
 
         manager->getWindowsToArrange(&w, &count);
@@ -905,7 +849,7 @@ void YWMApp::actionPerformed(YAction action, unsigned int /*modifiers*/) {
             delete [] w;
         }
     } else if (action == actionArrange) {
-        YFrameWindow **w = 0;
+        YFrameWindow **w = nullptr;
         int count = 0;
         manager->getWindowsToArrange(&w, &count);
         if (w) {
@@ -913,7 +857,7 @@ void YWMApp::actionPerformed(YAction action, unsigned int /*modifiers*/) {
             delete [] w;
         }
     } else if (action == actionHideAll || action == actionMinimizeAll) {
-        YFrameWindow **w = 0;
+        YFrameWindow **w = nullptr;
         int count = 0;
         manager->getWindowsToArrange(&w, &count);
         if (w) {
@@ -921,7 +865,7 @@ void YWMApp::actionPerformed(YAction action, unsigned int /*modifiers*/) {
             delete [] w;
         }
     } else if (action == actionShowDesktop) {
-        YFrameWindow **w = 0;
+        YFrameWindow **w = nullptr;
         int count = 0;
         manager->getWindowsToArrange(&w, &count, true, true);
         if (w && count > 0) {
@@ -933,7 +877,7 @@ void YWMApp::actionPerformed(YAction action, unsigned int /*modifiers*/) {
         }
         delete [] w;
     } else if (action == actionCascade) {
-        YFrameWindow **w = 0;
+        YFrameWindow **w = nullptr;
         int count = 0;
         manager->getWindowsToArrange(&w, &count);
         if (w) {
@@ -949,6 +893,14 @@ void YWMApp::actionPerformed(YAction action, unsigned int /*modifiers*/) {
             windowList->showFocused(-1, -1);
     } else if (action == actionWinOptions) {
         loadWinOptions(findConfigFile("winoptions"));
+    } else if (action == actionReloadKeys) {
+        keyProgs.clear();
+        MenuLoader(this, this, this).loadMenus(findConfigFile("keys"), nullptr);
+        if (manager && !initializing) {
+            if (manager->wmState() == YWindowManager::wmRUNNING) {
+                manager->grabKeys();
+            }
+        }
     } else if (action == actionCollapseTaskbar && taskBar) {
         taskBar->handleCollapseButton();
         manager->focusLastWindow();
@@ -1059,6 +1011,50 @@ void YWMApp::initFocusMode() {
     }
 }
 
+void YWMApp::loadFocusMode() {
+    const char* focusMode = nullptr;
+    cfoption focus_prefs[] = {
+        OSV("FocusMode", &focusMode,
+            "Focus mode (0=custom, 1=click, 2=sloppy"
+            ", 3=explicit, 4=strict, 5=quiet)"),
+        OK0()
+    };
+
+    YConfig::findLoadConfigFile(this, focus_prefs, "focus_mode");
+    if (focusMode) {
+        static const struct {
+            FocusModels num;
+            const char* str;
+        } models[] = {
+            { FocusCustom,   "custom"   },
+            { FocusClick,    "click"    },
+            { FocusSloppy,   "sloppy"   },
+            { FocusExplicit, "explicit" },
+            { FocusStrict,   "strict"   },
+            { FocusQuiet,    "quiet"    },
+        };
+        if (ASCII::isDigit(*focusMode)) {
+            int mode = atoi(focusMode);
+            for (int i = 0; i < int ACOUNT(models); ++i) {
+                if (mode == models[i].num) {
+                    this->focusMode = models[i].num;
+                    break;
+                }
+            }
+        }
+        else {
+            mstring mode(mstring(focusMode).lower());
+            for (int i = 0; i < int ACOUNT(models); ++i) {
+                if (mode == models[i].str) {
+                    this->focusMode = models[i].num;
+                    break;
+                }
+            }
+        }
+        delete[] const_cast<char *>(focusMode);
+    }
+}
+
 static void showExtensions() {
     struct {
         const char* str;
@@ -1070,6 +1066,7 @@ static void showExtensions() {
         { "render",    &render    },
         { "shapes",    &shapes    },
         { "xrandr",    &xrandr    },
+        { "xinerama",  &xinerama  },
     };
     printf("[name]   [ver] [ev][err]\n");
     for (int i = 0; i < int ACOUNT(xs); ++i) {
@@ -1093,7 +1090,7 @@ static int restartWM(const char* displayName, const char* overrideTheme) {
             WMConfig::setDefaultTheme(overrideTheme);
         }
         XClientMessageEvent message = {
-            ClientMessage, 0UL, False, 0, DefaultRootWindow(display),
+            ClientMessage, 0UL, False, nullptr, DefaultRootWindow(display),
             XInternAtom(display, "_ICEWM_ACTION", False), 32,
         };
         message.data.l[0] = CurrentTime;
@@ -1116,16 +1113,17 @@ YWMApp::YWMApp(int *argc, char ***argv, const char *displayName,
                 const char *configFile, const char *overrideTheme) :
     YSMApplication(argc, argv, displayName),
     mainArgv(*argv),
+    mainArgc(*argc),
     configFile(configFile),
     notifyParent(notifyParent),
     notifiedParent(0),
-    fLogoutMsgBox(0),
-    aboutDlg(0),
-    ctrlAltDelete(0),
-    switchWindow(0),
-    windowMenu(0),
+    fLogoutMsgBox(nullptr),
+    aboutDlg(nullptr),
+    ctrlAltDelete(nullptr),
+    switchWindow(nullptr),
+    windowMenu(nullptr),
     errorRequestCode(0),
-    errorFrame(0),
+    errorFrame(nullptr),
     splashWindow(splash(splashFile)),
     focusMode(FocusClick),
     managerWindow(None)
@@ -1133,23 +1131,20 @@ YWMApp::YWMApp(int *argc, char ***argv, const char *displayName,
     wmapp = this;
 
     WMConfig::loadConfiguration(this, configFile);
-    if (themeName != 0) {
+    if (themeName != nullptr) {
         MSG(("themeName=%s", themeName));
 
-        WMConfig::loadThemeConfiguration(this, themeName);
-    }
-    {
-        int focusMode(this->focusMode);
-        cfoption focus_prefs[] = {
-            OIV("FocusMode", &focusMode, FocusCustom, FocusModelLast,
-                "Focus mode (0=custom, 1=click, 2=sloppy"
-                ", 3=explicit, 4=strict, 5=quiet)"),
-            OK0()
-        };
-
-        YConfig::findLoadConfigFile(this, focus_prefs, "focus_mode");
-        this->focusMode = FocusModels(focusMode);
+        bool ok = WMConfig::loadThemeConfiguration(this, themeName);
+        if (ok == false && strcmp(themeName, CONFIG_DEFAULT_THEME)) {
+            themeName = CONFIG_DEFAULT_THEME;
+            ok = WMConfig::loadThemeConfiguration(this, themeName);
+        }
+        if (ok == false && strpcmp(themeName, "default", "/")) {
+            themeName = "default/default.theme";
+            ok = WMConfig::loadThemeConfiguration(this, themeName);
+        }
     }
+    loadFocusMode();
     WMConfig::loadConfiguration(this, "prefoverride");
     if (focusMode != FocusCustom)
         initFocusMode();
@@ -1175,12 +1170,12 @@ YWMApp::YWMApp(int *argc, char ***argv, const char *displayName,
     catchSignal(SIGPIPE);
 
     actionPerformed(actionWinOptions, 0);
-    MenuLoader(this, this, this).loadMenus(findConfigFile("keys"), 0);
+    actionPerformed(actionReloadKeys, 0);
 
     initPointers();
 
     if (post_preferences)
-        WMConfig::printPrefs(focusMode, loggingEvents, synchronizeX11, splashFile);
+        WMConfig::printPrefs(focusMode, wmapp_preferences);
     if (show_extensions)
         showExtensions();
 
@@ -1188,13 +1183,12 @@ YWMApp::YWMApp(int *argc, char ***argv, const char *displayName,
 
     managerWindow = registerProtocols1(*argv, *argc);
 
-    desktop = manager = new YWindowManager(
-        this, this, this, 0, root());
-    PRECONDITION(desktop != 0);
+    manager = new YWindowManager(
+        this, this, this, nullptr, root());
+    PRECONDITION(desktop != nullptr);
 
     registerProtocols2(managerWindow);
 
-    initFontPath(this);
     initIcons();
     initIconSize();
     WPixRes::initPixmaps();
@@ -1266,17 +1260,16 @@ YWMApp::YWMApp(int *argc, char ***argv, const char *displayName,
 YWMApp::~YWMApp() {
     if (fLogoutMsgBox) {
         manager->unmanageClient(fLogoutMsgBox);
-        fLogoutMsgBox = 0;
+        fLogoutMsgBox = nullptr;
     }
     if (aboutDlg) {
         manager->unmanageClient(aboutDlg);
-        aboutDlg = 0;
+        aboutDlg = nullptr;
     }
 
-    delete switchWindow; switchWindow = 0;
-    termIcons();
-    delete ctrlAltDelete; ctrlAltDelete = 0;
-    delete taskBar; taskBar = 0;
+    delete switchWindow; switchWindow = nullptr;
+    delete ctrlAltDelete; ctrlAltDelete = nullptr;
+    delete taskBar; taskBar = nullptr;
 
     if (statusMoveSize)
         statusMoveSize = null;
@@ -1288,7 +1281,7 @@ YWMApp::~YWMApp() {
 
     if (windowMenu) {
         windowMenu->setShared(false);
-        delete windowMenu; windowMenu = 0;
+        delete windowMenu; windowMenu = nullptr;
     }
 
     // shared menus last
@@ -1364,7 +1357,7 @@ bool YWMApp::handleIdle() {
     if ((QLength(display()) >> qbits) > 0) {
         ++qbits;
     }
-    else if (taskBar == 0 && showTaskBar) {
+    else if (taskBar == nullptr && showTaskBar) {
         createTaskBar();
         busy = true;
     }
@@ -1465,8 +1458,10 @@ static void print_usage(const char *argv0) {
              "  -a, --alpha         Use a 32-bit visual for translucency.\n"
              "  -c, --config=FILE   Load preferences from FILE.\n"
              "  -t, --theme=FILE    Load theme from FILE.\n"
-             "  --splash=IMAGE      Briefly show IMAGE on startup.\n"
-             "  --postpreferences   Print preferences after all processing.\n");
+             "  -s, --splash=IMAGE  Briefly show IMAGE on startup.\n"
+             "  -p, --postpreferences  Print preferences after all processing.\n"
+             "  --trace=conf,icon   Trace paths used to load configuration.\n"
+             );
 
     printf(_("Usage: %s [OPTIONS]\n"
              "Starts the IceWM window manager.\n"
@@ -1508,7 +1503,7 @@ static void print_usage(const char *argv0) {
 }
 
 static void print_themes_list() {
-    themeName = 0;
+    themeName = nullptr;
     ref<YResourcePaths> res(YResourcePaths::subdirs(null, true));
     for (int i = 0; i < res->getCount(); ++i) {
         for (sdir dir(res->getPath(i)); dir.next(); ) {
@@ -1516,7 +1511,7 @@ static void print_themes_list() {
             if (thmp.dirExists()) {
                 for (sdir thmdir(thmp); thmdir.nextExt(".theme"); ) {
                     upath theme(thmdir.path() + thmdir.entry());
-                    puts(cstring(theme));
+                    puts(theme.string());
                 }
             }
         }
@@ -1529,8 +1524,9 @@ static void print_confdir(const char *name, const char *path) {
 }
 
 static void print_directories(const char *argv0) {
+    upath priv(YApplication::getPrivConfDir());
     printf(_("%s configuration directories:\n"), argv0);
-    print_confdir("PrivConfDir", YApplication::getPrivConfDir().string());
+    print_confdir("PrivConfDir", priv.string());
     print_confdir("CFGDIR", CFGDIR);
     print_confdir("LIBDIR", LIBDIR);
     print_confdir("LOCDIR", LOCDIR);
@@ -1562,12 +1558,12 @@ static void print_configured(const char *argv0) {
 #ifdef CONFIG_GDK_PIXBUF_XLIB
     " gdkpixbuf"
 #endif
-#ifdef CONFIG_GNOME_MENUS
-    " gnomemenus"
-#endif
 #ifdef CONFIG_I18N
     " i18n"
 #endif
+#ifdef CONFIG_LIBICONV
+    " libiconv"
+#endif
 #ifdef CONFIG_LIBJPEG
     " libjpeg"
 #endif
@@ -1598,9 +1594,6 @@ static void print_configured(const char *argv0) {
 #ifdef CONFIG_UNICODE_SET
     " unicodeset"
 #endif
-#ifdef HAVE_WORDEXP
-    " wordexp"
-#endif
 #ifdef CONFIG_XFREETYPE
     " xfreetype" QUOTE(CONFIG_XFREETYPE)
 #endif
@@ -1609,9 +1602,6 @@ static void print_configured(const char *argv0) {
 #endif
 #ifdef CONFIG_XRANDR
     " xrandr"
-#endif
-#ifdef CONFIG_RENDER
-    " xrender"
 #endif
     "\n";
     printf(_("%s configured options:%s\n"), argv0,
@@ -1619,23 +1609,42 @@ static void print_configured(const char *argv0) {
     exit(0);
 }
 
+static void loadStartup(const char* configFile)
+{
+    upath prefs(YApplication::locateConfigFile(configFile));
+    if (prefs.nonempty()) {
+        YConfig::loadConfigFile(wmapp_preferences, prefs);
+    }
+
+    YXApplication::alphaBlending |= alphaBlending;
+    YXApplication::synchronizeX11 |= synchronizeX11;
+    if (tracingModules && YTrace::tracingConf() == nullptr) {
+        YTrace::tracing(tracingModules);
+    }
+
+    upath theme(YApplication::locateConfigFile("theme"));
+    if (theme.nonempty()) {
+        unsigned last = ACOUNT(wmapp_preferences) - 2;
+        YConfig::loadConfigFile(wmapp_preferences + last, theme);
+    }
+}
+
 int main(int argc, char **argv) {
     YLocale locale;
     bool restart_wm(false);
     bool notify_parent(false);
-    const char* configFile(0);
-    const char* displayName(0);
-    const char* overrideTheme(0);
-    const char* splashFile(ICESPLASH);
+    const char* configFile(nullptr);
+    const char* displayName(nullptr);
+    const char* overrideTheme(nullptr);
 
     for (char ** arg = argv + 1; arg < argv + argc; ++arg) {
         if (**arg == '-') {
-            char *value(0);
+            char *value(nullptr);
             if (GetArgument(value, "c", "config", arg, argv+argc))
                 configFile = value;
             else if (GetArgument(value, "t", "theme", arg, argv+argc))
                 overrideTheme = value;
-            else if (is_long_switch(*arg, "postpreferences"))
+            else if (is_switch(*arg, "p", "postpreferences"))
                 post_preferences = true;
             else if (is_long_switch(*arg, "extensions"))
                 show_extensions = true;
@@ -1673,7 +1682,7 @@ int main(int argc, char **argv) {
                 YXApplication::alphaBlending = true;
             else if (GetArgument(value, "d", "display", arg, argv+argc))
                 displayName = value;
-            else if (GetLongArgument(value, "splash", arg, argv+argc))
+            else if (GetArgument(value, "s", "splash", arg, argv+argc))
                 splashFile = value;
             else if (GetLongArgument(value, "trace", arg, argv+argc))
                 YTrace::tracing(value);
@@ -1687,27 +1696,7 @@ int main(int argc, char **argv) {
 
     if (isEmpty(configFile))
         configFile = "preferences";
-
-    {
-        cfoption options[] = {
-            OBV("Alpha", &YXApplication::alphaBlending, "Alpha blending"),
-            OBV("Synchronize", &YXApplication::synchronizeX11, "Synchronize X11"),
-            OBV("LogEvents", &loggingEvents, "Event Logging"),
-            OSV("Splash", &splashFile, "Splash image"),
-            OSV("Theme", &themeName, "Theme name"),
-            OK0()
-        };
-        upath prefs(YApplication::locateConfigFile(configFile));
-        if (prefs.nonempty()) {
-            YConfig::loadConfigFile(options, prefs);
-        }
-        upath theme(YApplication::locateConfigFile("theme"));
-        if (theme.nonempty()) {
-            unsigned last = ACOUNT(options) - 2;
-            YConfig::loadConfigFile(options + last, theme);
-        }
-        alphaBlending = YXApplication::alphaBlending;
-    }
+    loadStartup(configFile);
 
     if (nonempty(overrideTheme))
         themeName = overrideTheme;
@@ -1721,7 +1710,7 @@ int main(int argc, char **argv) {
     int rc = app.mainLoop();
     app.signalGuiEvent(geShutdown);
     manager->unmanageClients();
-    unregisterProtocols();
+    app.unregisterProtocols();
     YIcon::freeIcons();
     WMConfig::freeConfiguration();
     defOptions = null;
@@ -1730,7 +1719,7 @@ int main(int argc, char **argv) {
 }
 
 void YWMApp::createTaskBar() {
-    if (showTaskBar && taskBar == 0) {
+    if (showTaskBar && taskBar == nullptr) {
         manager->lockWorkArea();
         taskBar = new TaskBar(this, manager, this, this);
         for (YFrameIter frame = manager->focusedIterator(); ++frame; ) {
@@ -1747,7 +1736,7 @@ void YWMApp::doLogout(RebootShutdown reboot) {
     if (!confirmLogout)
         logout();
     else {
-        if (fLogoutMsgBox == 0) {
+        if (fLogoutMsgBox == nullptr) {
             YMsgBox *msgbox = new YMsgBox(YMsgBox::mbOK|YMsgBox::mbCancel);
             fLogoutMsgBox = msgbox;
             msgbox->setTitle(_("Confirm Logout"));
@@ -1802,7 +1791,7 @@ void YWMApp::handleMsgBox(YMsgBox *msgbox, int operation) {
     if (msgbox == fLogoutMsgBox && fLogoutMsgBox) {
         if (fLogoutMsgBox) {
             manager->unmanageClient(fLogoutMsgBox);
-            fLogoutMsgBox = 0;
+            fLogoutMsgBox = nullptr;
         }
         if (operation == YMsgBox::mbOK) {
             logout();
@@ -1834,11 +1823,14 @@ void YWMApp::handleSMAction(WMAction message) {
         wmapp->actionPerformed(actionAbout, 0);
         break;
     case ICEWM_ACTION_SUSPEND:
-        YWindowManager::execAfterFork(suspendCommand);
+        wmapp->runCommand(suspendCommand);
         break;
     case ICEWM_ACTION_WINOPTIONS:
         wmapp->actionPerformed(actionWinOptions, 0);
         break;
+    case ICEWM_ACTION_RELOADKEYS:
+        wmapp->actionPerformed(actionReloadKeys, 0);
+        break;
     }
 }
 
@@ -1846,7 +1838,7 @@ class SplashWindow : public YWindow {
     ref<YImage> image;
 public:
     SplashWindow(ref<YImage> image, int depth, Visual* visual) :
-        YWindow(0, None, depth, visual),
+        YWindow(nullptr, None, depth, visual),
         image(image)
     {
         setToplevel(true);
@@ -1858,10 +1850,11 @@ public:
         xapp->sync();
     }
     void place() {
+        YRect geo(desktop->getScreenGeometry());
         int w = int(image->width());
         int h = int(image->height());
-        int x = (xapp->displayWidth() - w) / 2;
-        int y = (xapp->displayHeight() - h) / 2;
+        int x = geo.x() + (geo.width() - w) / 2;
+        int y = geo.y() + (geo.height() - h) / 2;
         setGeometry(YRect(x, y, w, h));
         GraphicsBuffer(this).paint();
     }
@@ -1886,7 +1879,7 @@ public:
 };
 
 YWindow* YWMApp::splash(const char* splashFile) {
-    YWindow* window(0);
+    YWindow* window(nullptr);
     if (splashFile && 4 < strlen(splashFile)) {
         upath path(findConfigFile(splashFile));
         if (path.nonempty()) {
