@@ -2,39 +2,30 @@
 #define __TASKBAR_H
 
 #include "yaction.h"
-#include "ybutton.h"
-#include "ymenu.h"
 #include "ytimer.h"
 #include "wmclient.h"
 #include "yxtray.h"
+#include "applet.h"
 
 class ObjectBar;
-#if CONFIG_APPLET_CPU_STATUS
-class CPUStatus;
-#endif
-#ifdef CONFIG_APPLET_NET_STATUS
-class NetStatus;
-#endif
+class ObjectButton;
+class MEMStatus;
+class CPUStatusControl;
+class NetStatusControl;
 class AddressBar;
+class KeyboardStatus;
+class MailBoxControl;
 class MailBoxStatus;
+class YButton;
 class YClock;
 class YApm;
+class TaskBarMenu;
 class TaskPane;
 class TrayPane;
+class AWorkspaces;
 class WorkspacesPane;
 class YXTray;
 class YSMListener;
-class IApp;
-
-class IAppletContainer {
-public:
-    virtual void relayout() = 0;
-    virtual void contextMenu(int x_root, int y_root) = 0;
-protected:
-    virtual ~IAppletContainer() {}
-};
-
-#ifdef CONFIG_TASKBAR
 class TaskBar;
 
 class EdgeTrigger: public YWindow, public YTimerListener {
@@ -42,6 +33,8 @@ public:
     EdgeTrigger(TaskBar *owner);
     virtual ~EdgeTrigger();
 
+    bool enabled() const;
+    void show();
     void startHide();
     void stopHide();
 
@@ -52,7 +45,7 @@ public:
     virtual bool handleTimer(YTimer *t);
 private:
     TaskBar *fTaskBar;
-    YTimer *fAutoHideTimer;
+    lazy<YTimer> fAutoHideTimer;
     bool fDoShow;
 };
 
@@ -67,6 +60,7 @@ public:
     TaskBar(IApp *app, YWindow *aParent, YActionListener *wmActionListener, YSMListener *smActionListener);
     virtual ~TaskBar();
 
+private:
     virtual void paint(Graphics &g, const YRect &r);
     virtual bool handleKey(const XKeyEvent &key);
     virtual void handleButton(const XButtonEvent &button);
@@ -75,99 +69,92 @@ public:
     virtual void handleEndDrag(const XButtonEvent &down, const XButtonEvent &up);
 
     virtual void handleCrossing(const XCrossingEvent &crossing);
-#if false
-    virtual bool handleTimer(YTimer *t);
-#endif
+    virtual void handleExpose(const XExposeEvent &expose) {}
 
-    virtual void actionPerformed(YAction *action, unsigned int modifiers);
+    virtual void actionPerformed(YAction action, unsigned int modifiers);
     virtual void handlePopDown(YPopupWindow *popup);
-    virtual void handleEndPopup(YPopupWindow *popup);
 
     void updateWMHints();
     void updateLocation();
-    void configure(const YRect &r);
+    virtual void configure(const YRect2 &r);
+    virtual void repaint();
 
-#ifdef CONFIG_APPLET_CLOCK
     YClock *clock() { return fClock; }
-#endif
 
+public:
     bool windowTrayRequestDock(Window w);
-    void setWorkspaceActive(long workspace, int active);
+    void setWorkspaceActive(long workspace, bool active);
+    void workspacesRepaint();
+    void workspacesUpdateButtons();
+    void workspacesRelabelButtons();
+    void keyboardUpdate(mstring keyboard);
 
     void removeTasksApp(YFrameWindow *w);
     class TaskBarApp *addTasksApp(YFrameWindow *w);
     void relayoutTasks();
 
-    WorkspacesPane *workspacesPane() const { return fWorkspaces; }
-
     void popupStartMenu();
     void popupWindowListMenu();
 
-    void popOut();
     void showAddressBar();
-    void showBar(bool visible);
+    void showBar();
     void handleCollapseButton();
 
-    AddressBar *addressBar() const { return fAddressBar; }
-    TaskPane *taskPane() const { return fTasks; }
-#ifdef CONFIG_TRAY
-    TrayPane *windowTrayPane() const { return fWindowTray; }
-#endif
-
-#ifdef CONFIG_GRADIENTS
-    virtual ref<YImage> getGradient() const { return fGradient; }
-#endif    
-
-    void contextMenu(int x_root, int y_root);
-
     void relayout() { fNeedRelayout = true; }
     void relayoutNow();
 
     void detachDesktopTray();
-    void trayChanged();
-    YXTray *netwmTray() { return fDesktopTray; }
 
     void relayoutTray();
     class TrayApp *addTrayApp(YFrameWindow *w);
     void removeTrayApp(YFrameWindow *w);
 
+    bool hidden() const { return fIsCollapsed | fIsHidden | !fIsMapped; }
     bool autoTimer(bool show);
     void updateFullscreen(bool fullscreen);
     Window edgeTriggerWindow() { return fEdgeTrigger->handle(); }
+    void switchToPrev();
+    void switchToNext();
+    void movePrev();
+    void moveNext();
+    void refresh();
 
 private:
+    void popOut();
+
+    AddressBar *addressBar() const { return fAddressBar; }
+    TaskPane *taskPane() const { return fTasks; }
+    TrayPane *windowTrayPane() const { return fWindowTray; }
+
+    virtual ref<YImage> getGradient() const { return fGradient; }
+    const YSurface& getSurface() const { return fSurface; }
+
+    void contextMenu(int x_root, int y_root);
+    void buttonUpdate();
+    void trayChanged();
+    YXTray *netwmTray() { return fDesktopTray; }
+
+private:
+    GraphicsBuffer fGraphics;
+    YSurface fSurface;
     TaskPane *fTasks;
 
-    YButton *fCollapseButton;
-#ifdef CONFIG_TRAY
+    ObjectButton *fCollapseButton;
     TrayPane *fWindowTray;
-#endif
-#ifdef CONFIG_APPLET_CLOCK
     YClock *fClock;
-#endif
-#ifdef CONFIG_APPLET_MAILBOX
-    MailBoxStatus **fMailBoxStatus;
-#endif
-#ifdef CONFIG_APPLET_CPU_STATUS
-    CPUStatus *fCPUStatus;
-#endif
-#ifdef CONFIG_APPLET_APM
+    KeyboardStatus *fKeyboardStatus;
+    MailBoxControl *fMailBoxStatus;
+    MEMStatus *fMEMStatus;
+    CPUStatusControl *fCPUStatus;
     YApm *fApm;
-#endif
-#ifdef CONFIG_APPLET_NET_STATUS
-    NetStatus **fNetStatus;
-#endif
+    NetStatusControl *fNetStatus;
 
-#ifndef NO_CONFIGURE_MENUS
     ObjectBar *fObjectBar;
-    YButton *fApplications;
-#endif
-#ifdef CONFIG_WINMENU
-    YButton *fWinList;
-#endif
-    YButton *fShowDesktop;
+    ObjectButton *fApplications;
+    ObjectButton *fWinList;
+    ObjectButton *fShowDesktop;
     AddressBar *fAddressBar;
-    WorkspacesPane *fWorkspaces;
+    AWorkspaces *fWorkspaces;
     YXTray *fDesktopTray;
     YActionListener *wmActionListener;
     YSMListener *smActionListener;
@@ -178,49 +165,38 @@ private:
     bool fIsCollapsed;
     bool fIsMapped;
     bool fMenuShown;
-#if false
-    YTimer *fAutoHideTimer;
-#endif
 
-    YMenu *taskBarMenu;
+    lazy<TaskBarMenu> taskBarMenu;
 
     friend class WindowList;
     friend class WindowListBox;
-    
-#ifdef CONFIG_GRADIENTS
+
     ref<YImage> fGradient;
-#endif
 
     bool fNeedRelayout;
+    bool fButtonUpdate;
 
-    void initMenu();
     void initApplets();
-    void updateLayout(int &size_w, int &size_h);
+    void updateLayout(unsigned &size_w, unsigned &size_h);
 
     EdgeTrigger *fEdgeTrigger;
+
+    class YStrut {
+    public:
+        Atom left, right, top, bottom;
+        YStrut() : left(0), right(0), top(0), bottom(0) { }
+        bool operator!=(const YStrut& s) const {
+            return left != s.left || right != s.right
+                || top != s.top || bottom != s.bottom;
+        }
+        const Atom* operator&() const { return &left; }
+    } fStrut;
 };
 
 extern TaskBar *taskBar; // !!! get rid of this
 
-#if 1
-extern ref<YPixmap> startPixmap;
-extern ref<YPixmap> windowsPixmap;
-
-extern ref<YPixmap> taskbackPixmap;
-extern ref<YPixmap> taskbuttonPixmap;
-extern ref<YPixmap> taskbuttonactivePixmap;
-extern ref<YPixmap> taskbuttonminimizedPixmap;
-#endif
-
-#ifdef CONFIG_GRADIENTS
-class YPixbuf;
-
-extern ref<YImage> taskbackPixbuf;
-extern ref<YImage> taskbuttonPixbuf;
-extern ref<YImage> taskbuttonactivePixbuf;
-extern ref<YImage> taskbuttonminimizedPixbuf;
-#endif
+extern YColorName taskBarBg;
 
 #endif
 
-#endif
+// vim: set sw=4 ts=4 et:
