@@ -5,8 +5,6 @@
  */
 #include "config.h"
 
-#ifdef CONFIG_WINMENU
-
 #include "wmaction.h"
 #include "ylib.h"
 #include "ymenu.h"
@@ -19,29 +17,28 @@
 #include "wmmgr.h"
 #include "wmframe.h"
 #include "wmwinmenu.h"
+#include "workspaces.h"
 
 #include "intl.h"
 
-class ActivateWindowMenuItem: public YMenuItem, public YAction {
+class ActivateWindowMenuItem: public YMenuItem {
 public:
-    ActivateWindowMenuItem(YFrameWindow *frame): 
-        YMenuItem(frame->getTitle(), -1, null, this, 0),
+    ActivateWindowMenuItem(YFrameWindow *frame):
+        YMenuItem(frame->getTitle(), -1, null, YAction(), nullptr),
         fFrame(frame)
     {
-#ifndef LITE
         if (fFrame->clientIcon() != null)
             setIcon(fFrame->clientIcon());
-#endif
     }
 
-    virtual void actionPerformed(YActionListener * /*listener*/, YAction * /*action*/, unsigned int modifiers) {
+    virtual void actionPerformed(YActionListener * /*listener*/, YAction /*action*/, unsigned int modifiers) {
         YFrameWindow *f = manager->topLayer();
 
         while (f) {
             if ((void *)f == fFrame) {
                 if (modifiers & ShiftMask)
                     f->wmOccupyOnlyWorkspace(manager->activeWorkspace());
-                manager->activate(f, true);
+                f->activateWindow(true, false);
                 return ;
             }
             f = f->nextLayer();
@@ -63,7 +60,7 @@ YMenu *YWindowManager::createWindowMenu(YMenu *menu, long workspace) {
     /// !!! fix performance (smarter update, on change only)
     for (int layer = 0 ; layer < WinLayerCount; layer++) {
         layerCount = 0;
-        if (top(layer) == 0)
+        if (top(layer) == nullptr)
             continue;
         for (level = 0; level < 4; level++) {
             levelCount = 0;
@@ -72,10 +69,8 @@ YMenu *YWindowManager::createWindowMenu(YMenu *menu, long workspace) {
                     continue;
                 if (!frame->visibleOn(workspace))
                     continue;
-#ifndef NO_WINDOW_OPTIONS
-                if (frame->frameOptions() & YFrameWindow::foIgnoreWinList)
+                if (frame->frameOption(YFrameWindow::foIgnoreWinList))
                     continue;
-#endif
                 if (workspace != activeWorkspace() &&
                     frame->visibleOn(activeWorkspace()))
                     continue;
@@ -91,7 +86,7 @@ YMenu *YWindowManager::createWindowMenu(YMenu *menu, long workspace) {
                 if (level != windowLevel)
                     continue;
 
-                if ((levelCount == 0 && level > 0) || 
+                if ((levelCount == 0 && level > 0) ||
                     ((layerCount == 0 && layer > 0) && needSeparator))
                     menu->addSeparator();
 
@@ -105,7 +100,10 @@ YMenu *YWindowManager::createWindowMenu(YMenu *menu, long workspace) {
     return menu;
 }
 
-WindowListMenu::WindowListMenu(YWindow *parent): YMenu(parent) {
+WindowListMenu::WindowListMenu(YActionListener *app, YWindow *parent):
+    YMenu(parent)
+{
+    setActionListener(app);
 }
 
 void WindowListMenu::updatePopup() {
@@ -115,26 +113,30 @@ void WindowListMenu::updatePopup() {
 
     bool first = true;
 
-    for (long d = 0; d < manager->workspaceCount(); d++) {
+    for (int d = 0; d < workspaceCount; d++) {
         if (d == manager->activeWorkspace())
             continue;
         if (first) {
             addSeparator();
             first = false;
         }
-        char s[50];
-        sprintf(s, _("%lu. Workspace %-.32s"), (unsigned long)(d + 1),
-                manager->workspaceName(d));
+        char s[128];
+        snprintf(s, sizeof s,
+                _("%lu. Workspace %-.32s"), (unsigned long)(d + 1),
+                workspaceNames[d]);
 
-        YMenu *sub = 0;
+        YMenu *sub = nullptr;
         if (manager->windowCount(d) > 0) // !!! do lazy create menu instead
-            sub = manager->createWindowMenu(0, d);
+            sub = manager->createWindowMenu(nullptr, d);
         addItem(s, (d < 10) ? 0 : -1, workspaceActionActivate[d], sub);
     }
-#ifdef CONFIG_WINLIST
     addSeparator();
     addItem(_("_Window list"), -2, KEY_NAME(gKeySysWindowList), actionWindowList);
-#endif
 }
 
-#endif
+void WindowListMenu::activatePopup(int flags) {
+    super::activatePopup(flags);
+    repaint();
+}
+
+// vim: set sw=4 ts=4 et:
